
import email

def getUnicodeEmail(filename):
	"""
	Take a Japanese e-mail file, decode to UTF-8.
	Currently supports shift_jis, iso-2022, and utf-8, possibly more.
	Currently only supports a single text/plain, no other content types will 
	be attempted to be returned.
	"""
	#a = file(filename).read()
	b = file(filename)
	msg = email.message_from_file(b)
	d = ''
	
	for part in msg.walk():
	
		ctype = part.get_content_type()
		#print ctype
		if ctype == 'text/plain':
			d = str(part)
			#d == a for this one, darn..
	
	#next part
	
	ctype = msg['Content-Type']
	# ctype = 'text/plain;\n  charset="SHIFT_JIS"'
	charset = ctype[ctype.find('charset=')+8:].replace('"', '')
	
	if charset == '':
		# It's unlikely that this works, but oh well, better to return bad 
		# data than to return nothing.
		return d
	#end if
	
	import codecs
	sjisd = codecs.getdecoder(charset)
	readable, size = sjisd(d)
	# Now we're in bizness.
	return readable
#end def getUnicodeEmail()

def addHistogram(hist1, hist2):
	"""
	Return the addition of two histograms,
	assuming the first is larger.
	"""
	# Assume hist1 is going to have more.
	output = dict(hist1)
	for k in hist2.keys():
		if k in hist1:
			output[k] += hist2[k]
		else:
			output[k] = hist2[k]
		#end if
	#next k
	return output
#end def addHistogram(hist1, hist2)

def pwnHistogram(filename = 'spam20080925/histogram/tot_hist3.txt'):
	"""
	A histogram is a very specific file format I designed.
	Example:
	# title
	1234: 5
	
	"""
	a = file(filename).read()
	b = a.split('\n')
	# First line is title.
	b.pop(0)
	# Last line is tailing newline.
	b.pop()
	output = {}
	for line in b:
		cord, count = line.split(': ')
		output[unichr(int(cord))] = count
	#next line
	return output
#end def pwnHistogram(filename)

def main():
	"""
	Take a list of Japanese e-mail files, decode to UTF-8.
	Do a histogram on each seperately.
	
	"""
	from sys import argv
	files = ['spam20080925/1212037675.9396.rK9C7']
	showHistogram = False
	showUtf8 = False
	showPwnedHistogram = False
	if len(argv) > 1:
		# --histogram, --utf8
		if '--histogram' in argv:
			argv.pop(argv.index('--histogram'))
			showHistogram = True
		#end if
		if '--utf8' in argv:
			argv.pop(argv.index('--utf8'))
			showUtf8 = True
		#end if
		if '--pwnhistogram' in argv:
			argv.pop(argv.index('--pwnhistogram'))
			showPwnedHistogram = True
		#end if
		files = argv[1:]
	#end if
	
	# if they don't give any option, give them histogram and utf8 (default)
	if not showHistogram and not showUtf8 and not showPwnedHistogram:
			showHistogram = True
			showUtf8 = True
	#end if
	
	if showPwnedHistogram:
		# Utility function pwns a histogram.
		for filename in files:
			print prettyDict(pwnHistogram(filename)).encode('utf-8')
		#next filename
		return
	#end if
	
	tot_histogram = {}
	
	for filename in files:
		print "# R0x0ring ", filename
		try:
			readable = getUnicodeEmail(filename)
			if showUtf8: print readable.encode('utf-8')
		except Exception, e:
			print "# Exception in", filename, e
			continue
		#end try
		
		from sys import path
		path.append('/home/jvoss/scripts')
		import histogram1
		hist1 = histogram1.unicodeHistogram(readable)
		if showHistogram: print prettyDict(hist1)
		tot_histogram = addHistogram(tot_histogram, hist1)
	#next filename
	
	if len(files) > 1:
		print 'Total Histogram:'
		print prettyDict(tot_histogram)
	#end if
	
#end def main()

def prettyDict(d):
	# This is pretty wild, sorts the keys.
	e = [(k,d[k]) for k in sorted(d.keys())]
	return u'\n'.join([unicode(k) + u': ' + unicode(v) for (k, v) in e])
#end def prettyListDict(ld)

if __name__ == '__main__':
	main()
#end if
